<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class ControleController extends Controller
{
    private $baseUrl;
    private $apikey;
    private $queueId;
    protected $tabelaPreco;

    public function __construct(TabelaPreco $tabelaPreco)
    {
        $this->apikey = (string) env('EVOTALKS_API_KEY');
        $this->baseUrl = (string) env('EVOTALKS_BASE_URL');
        $this->queueId = (int) env('EVOTALKS_API_QUEUEID');
        $this->tabelaPreco = $tabelaPreco;
    }

    private function apiHeaders()
    {
        return [
            'Accept' => 'application/json',
            'Content-Type' => 'application/json',
        ];
    }

    private function sendPostRequest($endpoint, $data)
    {
        $response = Http::withOptions(['verify' => false])
            ->withHeaders($this->apiHeaders())
            ->post($this->baseUrl . $endpoint, $data);

        if ($response->successful()) {
            return $response;
        }

        Log::error('Erro na requisição ao EvoTalks', [
            'endpoint' => $endpoint,
            'status' => $response->status(),
            'body' => $response->body(),
        ]);

        return $response;
    }

    public function addContact(array $contactData)
    {
        Log::info('Cadastrando contato no EvoTalks', ['data' => $contactData]);

        $response = $this->sendPostRequest('/int/addContact', $contactData);

        if ($response->successful()) {
            $responseData = $response->json();
            Log::info('Contato criado com sucesso no EvoTalks', ['id' => $responseData['contactId'] ?? null]);
            return $responseData;
        }

        Log::error('Erro ao criar contato no EvoTalks', [
            'status' => $response->status(),
            'body' => $response->body(),
        ]);

        return null;
    }

    private function mapProducts($expectedOperations)
    {
        $mappedProducts = [];

        $mappedProducts[0] = [
            'id'        => $this->tabelaPreco->getTaxaOperacao($expectedOperations)['id'],
            'qty'       => $expectedOperations,
            'discount'  => 0,
        ];

        $mappedProducts[1] = [
            'id'        => 2,
            'qty'       => 0,
            'discount'  => 0,
        ];

        Log::info('Produtos mapeados: ', $mappedProducts);

        return $mappedProducts;
    }

    public function receiveOpportunity(Request $request)
    {
        Log::info('Recebimento do Controle: ', ['JSON: ', $request]);

        $validated = $request->validate([
            'imobiaCode'            => 'required|numeric',
            'name'                  => 'required|string',
            'document'              => 'nullable|string',
            'phoneNumber'           => 'required|string',
            'email'                 => 'nullable|email',
            'address'               => 'nullable|string',
            'houseNumber'           => 'nullable|string',
            'city'                  => 'nullable|string',
            'state'                 => 'nullable|string',
            'country'               => 'nullable|string',
            'postalCode'            => 'nullable|string',
            'modules'               => 'nullable|array',
            'expectedOperations'    => 'nullable|numeric',
            'value'                 => 'nullable|numeric',
        ]);

        // Dados do contato
        $contactData = [
            'queueId'       => $this->queueId,
            'apiKey'        => $this->apikey,
            'name'          => $validated['imobiaCode'] . ' - ' . $validated['name'],
            'document'      => $validated['document'] ?? null,
            'number'        => $validated['phoneNumber'],
            'email'         => $validated['email'] ?? null,
            'address'       => $validated['address'] ?? null,
            'houseNumber'   => $validated['houseNumber'] ?? null,
            'city'          => $validated['city'] ?? null,
            'state'         => $validated['state'] ?? null,
            'country'       => $validated['country'] ?? null,
            'postalCode'    => $validated['postalCode'] ?? null,
            'tag'           => [1]
        ];

        $contactResponse = $this->addContact($contactData);

        if (!$contactResponse) {
            return response()->json(['error' => 'Erro ao cadastrar contato no EvoTalks'], 500);
        }

        // Dados da oportunidade
        $opportunityData = [
            'queueId'       => $this->queueId,
            'apiKey'        => $this->apikey,
            'fkPipeline'    => 3,
            'fkStage'       => 16,
            'responsableid' => 507,
            'title'         => $validated['imobiaCode'] . ' - ' . $validated['name'],
            'clientid'      => (string) $validated['imobiaCode'],
            'mainphone'     => $validated['phoneNumber'],
            'mainmail'      => $validated['email'] ?? '',
            'description'   => 'Módulos ativos: ' . implode(', ', $validated['modules']),
            'value'         => number_format($validated['value'] ?? 0, 2, '.', ''),
            'contacts'      => [$contactResponse['contactId']],
            'products'      => $validated['expectedOperations'] == 0 & $validated['value'] == 19.90 ? $this->mapProducts(1) : $this->mapProducts($validated['expectedOperations'])
        ];

        // Limpar campos vazios
        foreach ($opportunityData as $key => $value) {
            if (empty($value) && !is_numeric($value)) {
                unset($opportunityData[$key]);
            }
        }

        Log::info('Cadastrando oportunidade no EvoTalks', ['data' => $opportunityData]);

        $response = $this->sendPostRequest('/int/createOpportunity', $opportunityData);

        if ($response->successful()) {
            $responseData = $response->json();
            Log::info('Oportunidade criada com sucesso no EvoTalks', ['id' => $responseData['id'] ?? null]);
            return response()->json($responseData, 201);
        }

        Log::error('Erro ao criar oportunidade no EvoTalks', [
            'status' => $response->status(),
            'body' => $response->body(),
        ]);

        return response()->json(['error' => 'Erro ao criar oportunidade no EvoTalks'], $response->status());
    }
}
