<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class EvoTalksController extends Controller
{
    private $baseUrl;
    private $apikey;
    private $queueId;

    public function __construct()
    {
        $this->apikey = (string) env('EVOTALKS_API_KEY');
        $this->baseUrl = (string) env('EVOTALKS_BASE_URL');
        $this->queueId = (int) env('EVOTALKS_API_QUEUEID');
    }

    private function apiHeaders()
    {
        return [
            'Accept' => 'application/json',
            'Content-Type' => 'application/json',
        ];
    }

    private function requestBody(Request $request)
    {
        return array_merge([
            'queueId' => $this->queueId,
            'apiKey' => $this->apikey,
        ], $request->all());
    }

    public function createOpportunity(Request $request)
    {
        $data = $this->requestBody($request);

        Log::info('Enviando requisição para EvoTalks', ['data' => $data]);

        $response = $this->sendPostRequest('/int/createOpportunity', $data);

        if ($response->successful()) {
            $responseData = $response->json();
            Log::info('Oportunidade criada com sucesso no EvoTalks', ['id' => $responseData['id'] ?? null]);

            return response()->json($responseData);
        }

        Log::error('Erro ao criar oportunidade no EvoTalks', [
            'status' => $response->status(),
            'body' => $response->body(),
        ]);

        return response()->json([
            'error' => 'Erro ao criar oportunidade no EvoTalks',
            'details' => $response->body(),
        ], $response->status());
    }

    public function addNotesToOpportunity(Request $request)
    {
        $validated = $request->validate([
            'opportunity_id' => 'required|integer',
            'notes' => 'required|string',
        ]);

        Log::info('Adicionando notas à oportunidade no EvoTalks', ['data' => $validated]);

        try {
            $response = $this->sendPostRequest('/int/insertOpportunityNote', [
                'queueId' => $this->queueId,
                'apiKey' => $this->apikey,
                'id' => $validated['opportunity_id'],
                'note' => $validated['notes'],
            ]);

            if ($response->successful()) {
                Log::info('Notas adicionadas com sucesso no EvoTalks', ['opportunity_id' => $validated['opportunity_id']]);
                return response()->json(['success' => true, 'message' => 'Notas adicionadas com sucesso']);
            }

            Log::error('Erro ao adicionar notas no EvoTalks', [
                'opportunity_id' => $validated['opportunity_id'],
                'status' => $response->status(),
                'body' => $response->body(),
            ]);

            return response()->json([
                'error' => 'Erro ao adicionar notas no EvoTalks',
                'details' => $response->body(),
            ], $response->status());
        } catch (\Exception $e) {

            Log::error('Exceção ao adicionar notas no EvoTalks', ['error' => $e->getMessage()]);

            return response()->json([
                'error' => 'Erro interno ao processar a solicitação',
            ], 500);
        }
    }

    public function createTask(Request $request)
    {
        $data = $this->requestBody($request);

        Log::info('Criando tarefa no EvoTalks', ['data' => $data]);

        $response = $this->sendPostRequest('/int/createTask', $data);

        if ($response->successful()) {
            Log::info('Tarefa criada com sucesso no EvoTalks', ['response' => $response->json()]);
            return $response->json();
        } else {
            Log::error('Erro ao criar tarefa no EvoTalks', [
                'status' => $response->status(),
                'body' => $response->body(),
            ]);
            return response()->json([
                'error' => 'Falha ao criar tarefa no EvoTalks',
                'details' => $response->json(),
            ], $response->status());
        }
    }

    public function markAsDone($idTask, $data)
    {
        $data = [
            'queueId' => $this->queueId,
            'apiKey' => $this->apikey,
            'id' => $idTask,
            'userId' => 1,
        ];

        Log::info('Buscando tarefa para marcar como concluído', []);

        $response = $this->sendPostRequest('/int/markAsDone', $data);

        if ($response->successful()) {
            Log::info('Tarefa buscada com sucesso', ['response' => $response->json()]);
            return response()->json($response->json());
        } else {
            Log::error('Erro ao buscar Tarefa no EvoTalks', [
                'status' => $response->status(),
                'body' => $response->body(),
            ]);
            return response()->json([
                'error' => 'Falha ao buscar Tarefa no EvoTalks',
                'details' => $response->json(),
            ], $response->status());
        }
    }

    public function addContact($contactData)
    {
        $data = array_merge([
            'queueId' => $this->queueId,
            'apiKey'  => $this->apikey,
        ], $contactData);    

        Log::info('Cadastrando contato no EvoTalks', ['data' => $data]);

        $response = $this->sendPostRequest('/int/addContact', $data);

        if ($response->successful()) {
            $responseData = $response->json();
            Log::info('Contato criado com sucesso no EvoTalks', ['id' => $responseData['contactId'] ?? null]);
            return $responseData;
        }

        Log::error('Erro ao criar contato no EvoTalks', [
            'status' => $response->status(),
            'body' => $response->body(),
        ]);

        return null;
    }

    private function sendPostRequest($endpoint, $data)
    {
        $response = Http::withOptions(['verify' => false])
            ->withHeaders($this->apiHeaders())
            ->post($this->baseUrl . $endpoint, $data);

        if ($response->successful()) {
            return $response;
        }

        Log::error('Erro na requisição ao EvoTalks', [
            'endpoint' => $endpoint,
            'status' => $response->status(),
            'body' => $response->body(),
        ]);

        return $response;
    }
}
