<?php

namespace App\Http\Controllers;

use App\Helper\PipeDriveToEvoTalksMapper;
use App\Helper\TabelaPreco;
use App\Services\PipeStage;
use App\Services\syncPipeDriveContactToEvoTalks;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class SyncController extends Controller
{
    protected $pipeDriveController;
    protected $evoTalksController;
    protected $tabelaPreco;
    protected $pipeStage;
    protected $pipeDriveToEvoTalksMapper;
    protected $syncPipeDriveContactToEvoTalks;

    public function __construct(
        PipeDriveController $pipeDriveController, 
        EvoTalksController $evoTalksController, 
        TabelaPreco $tabelaPreco, 
        PipeStage $pipeStage,
        PipeDriveToEvoTalksMapper $pipeDriveToEvoTalksMapper,
        syncPipeDriveContactToEvoTalks $syncPipeDriveContactToEvoTalks
    )
    {
        $this->pipeDriveController = $pipeDriveController;
        $this->evoTalksController = $evoTalksController;
        $this->tabelaPreco = $tabelaPreco;
        $this->pipeStage = $pipeStage;
        $this->pipeDriveToEvoTalksMapper = $pipeDriveToEvoTalksMapper;
        $this->syncPipeDriveContactToEvoTalks = $syncPipeDriveContactToEvoTalks;
    }

    public function syncCreatedOpportunityFromPipeDrive(Request $request)
    {
        Log::info('Recebendo webhook do Pipedrive', ['payload' => $request->all()]);

        $pipeDriveData = $request->input('data');
        //$pipeDriveData = $request->all();
        Log::info('ID oportunidade: ', [$pipeDriveData['id']]);
        $data = $this->pipeDriveController->getDeal($pipeDriveData['id']);
        $dealV2 = $this->pipeDriveController->getDealV2($pipeDriveData['id']);

        if (!$data) {
            Log::error("Falha ao recuperar dados do negócio com ID {$pipeDriveData['id']}.");
            return response()->json(['error' => 'Failed to retrieve deal data'], 500);
        }

        if (!$this->isValidPayload($data) || ($data['status'] ?? '') !== 'won') {
            Log::warning('Payload inválido ou oportunidade não marcada como "ganha"', ['payload' => $data]);
            return response()->json(['error' => 'Invalid or non-won opportunity'], 400);
        }

        $fkPipeline = $this->pipeStage->getPipelineId($data['pipeline_id'], $dealV2['label_ids'] == [] ? [] : $dealV2['label_ids']);
        if (!$fkPipeline) {
            Log::warning('Pipeline não permitido', ['pipeline_id' => $data['pipeline_id']]);
            return response()->json(['error' => 'Pipeline not allowed'], 403);
        }

        $fkStage = $this->pipeStage->getStageId($fkPipeline, $data['stage_id']);
        $contact = $this->syncPipeDriveContactToEvoTalks->syncPipeDriveContactToEvoTalks($data);
        $evoData = $this->pipeDriveToEvoTalksMapper->mapPipeDriveToEvoTalks($data, $dealV2, $fkPipeline, $fkStage, $contact);
        $response = $this->evoTalksController->createOpportunity($evoData);
        Log::info('Mapeando dados para EvoTalks', ['pipeline_id' => $data['pipeline_id'], 'stage_id' => $fkStage]);
        Log::info('Enviando dados para criação de oportunidade no EvoTalks', ['evoData' => $evoData->all()]);

        if ($response->status() === 200 && isset($response->getData()->id)) {

            $opportunityId = $response->getData()->id;
            $notes = $this->pipeDriveController->fetchNotes($data['id']);
            
            if ($notes) {
                Log::info("Adicionando notas à oportunidade no EvoTalks", ['opportunity_id' => $opportunityId, 'notes' => $notes]);

                $this->evoTalksController->addNotesToOpportunity(new Request([
                    'opportunity_id' => $opportunityId,
                    'notes' => $notes,
                ]));
            }

            $this->syncActivitiesFromPipeDrive(new Request(['deal_id' => $data['id']]), $opportunityId);
        } else {
            Log::error('Erro ao criar oportunidade no EvoTalks', ['response' => $response->json()]);
        }

        return $response;
    }

    public function syncActivitiesFromPipeDrive(Request $request, $opportunityId)
    {
        Log::info('Iniciando sincronização de atividades do Pipedrive para o EvoTalks');
        $dealId = $request->input('deal_id');

        if (!$dealId) {
            Log::warning('ID do negócio (deal_id) não fornecido');
            return response()->json(['error' => 'deal_id is required'], 400);
        }

        $activities = $this->pipeDriveController->fetchDealFlowActivities($dealId);
        if (empty($activities)) {
            Log::info('Nenhuma atividade do tipo "activity" encontrada no Pipedrive', ['deal_id' => $dealId]);
            return response()->json(['message' => 'No activities found for the given deal'], 204);
        }

        foreach ($activities as $activity) {
            $this->syncPipeDriveActivitiesToEvoTalks($activity, $opportunityId);
        }

        return response()->json(['message' => 'Activities synced successfully'], 200);
    }

    private function isValidPayload($data)
    {
        return is_array($data) && isset($data['id'], $data['status'], $data['pipeline_id']);
    }  
}
