<?php

namespace App\Services;

use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Log;

class PipeDriveService
{
    private $baseUrl;
    private $apiToken;

    public function __construct()
    {
        $this->apiToken = env('PIPEDRIVE_API_TOKEN');
        $this->baseUrl = env('PIPEDRIVE_BASE_URL');
    }

    public function getDealV2($id)
    {
        $url = "{$this->baseUrl}/api/v2/deals/{$id}";
        $params = ['api_token' => $this->apiToken];

        $response = Http::get($url, $params);

        if ($response->successful()) {
            return $response->json()['data'];
        }

        Log::error("Failed to retrieve deal V2 from Pipedrive", [
            'deal_id' => $id,
            'response' => $response->body(),
        ]);

        return null; // Retorna null para representar falha
    }

    public function getDeal($id)
    {
        $url = "{$this->baseUrl}/v1/deals/{$id}";
        $params = ['api_token' => $this->apiToken];

        $response = Http::get($url, $params);

        if ($response->successful()) {
            return $response->json()['data'];
        }

        Log::error("Failed to retrieve deal from Pipedrive", [
            'deal_id' => $id,
            'response' => $response->body(),
        ]);

        return null; // Retorna null para representar falha
    }

    public function fetchNotes($dealId): string
    {
        $url = "{$this->baseUrl}/v1/notes?deal_id={$dealId}&api_token={$this->apiToken}";

        try {
            $response = Http::get($url);

            if ($response->successful() && isset($response->json()['data'])) {
                $notes = $response->json()['data'];
                $formattedNotes = '';

                foreach ($notes as $note) {
                    $content = $note['content'] ?? 'Sem conteúdo';
                    $formattedNotes .= "- {$content}\n";

                    if (strpos($content, 'importante') !== false) {
                        Log::info("Nota marcada como importante: {$content}");
                    }
                }

                return trim($formattedNotes); // Remove espaços em branco finais
            }
        } catch (\Exception $e) {
            Log::error("Erro ao buscar notas para o negócio {$dealId}", ['message' => $e->getMessage()]);
        }

        return '';
    }

    public function fetchDealFlowActivities($dealId): array
    {
        $url = "{$this->baseUrl}/v1/deals/{$dealId}/flow?api_token={$this->apiToken}";

        try {
            $response = Http::get($url);

            if ($response->successful() && isset($response->json()['data'])) {
                $flowData = $response->json()['data'];

                $activities = array_filter($flowData, function ($entry) {
                    return $entry['object'] === 'activity';
                });

                Log::info('Atividades filtradas do Pipedrive', ['activities' => $activities]);
                return $activities;
            }
        } catch (\Exception $e) {
            Log::error("Erro ao buscar atividades do Pipedrive para o negócio {$dealId}", ['message' => $e->getMessage()]);
        }

        return [];
    }
}
