<?php

namespace App\Http\Controllers;

use App\Helper\PipeDriveToEvoTalksMapper;
use App\Helper\TabelaPreco;
use App\Services\PipeStage;
use App\Services\SyncPipeDriveContactToEvoTalks;
use App\Services\SyncPipeDriveActivitiesToEvoTalks;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Http;
use App\Helper\SlackNotifier;

class SyncController extends Controller
{
    protected $pipeDriveController;
    protected $evoTalksController;
    protected $tabelaPreco;
    protected $pipeStage;
    protected $pipeDriveToEvoTalksMapper;
    protected $syncPipeDriveContactToEvoTalks;
    protected $syncPipeDriveActivitiesToEvoTalks;

    public function __construct(
        PipeDriveController                 $pipeDriveController, 
        EvoTalksController                  $evoTalksController, 
        TabelaPreco                         $tabelaPreco, 
        PipeStage                           $pipeStage,
        PipeDriveToEvoTalksMapper           $pipeDriveToEvoTalksMapper,
        SyncPipeDriveContactToEvoTalks      $syncPipeDriveContactToEvoTalks,
        SyncPipeDriveActivitiesToEvoTalks   $syncPipeDriveActivitiesToEvoTalks
    )
    {
        $this->pipeDriveController                  = $pipeDriveController;
        $this->evoTalksController                   = $evoTalksController;
        $this->tabelaPreco                          = $tabelaPreco;
        $this->pipeStage                            = $pipeStage;
        $this->pipeDriveToEvoTalksMapper            = $pipeDriveToEvoTalksMapper;
        $this->syncPipeDriveContactToEvoTalks       = $syncPipeDriveContactToEvoTalks;
        $this->syncPipeDriveActivitiesToEvoTalks    = $syncPipeDriveActivitiesToEvoTalks;
    }

    public function syncCreatedOpportunityFromPipeDrive(Request $request)
    {
        Log::info('info', ['Recebendo oportunidade do Pipe para o Evo' => $request->all()]);
        
        // exit('Execução interrompida');
        // $pipeDriveData = $request->all();
        
        $pipeDriveData = $request->input('data');
        $data = $this->pipeDriveController->getDeal($pipeDriveData['id']);
        $dataV2 = $this->pipeDriveController->getDealV2($pipeDriveData['id']);

        if (!$this->isValidPayload($dataV2) || ($dataV2['status'] ?? '') !== 'won') {
            Log::warning('Payload inválido ou oportunidade não marcada como "ganha"', ['payload' => $data['title']]);
            return response()->json(['Invalid or non-won opportunity'], 200);
        }

        if (!$data) {
            Log::info('Falha ao recuperar dados do negócio com ID {$pipeDriveData["id"]}.');
            return response()->json(['error' => 'Failed to retrieve deal data'], 500);
        }
        
        $fkPipeline = $this->pipeStage->getPipelineId($data['pipeline_id'], $dataV2['label_ids'] == [] ? [] : $pipeDriveData['label_ids']);
        if (!$fkPipeline || $fkPipeline != 1) {
            Log::info('Pipeline não permitido', ['pipeline_id' => $data['pipeline_id']]);
            return response()->json(['error' => 'Pipeline not allowed'], 403);
        }
        
        SlackNotifier::sendNotification('info', 'Recebendo oportunidade do Pipe para o Evo', ['Client: ' => $request->input('data')['title'], 'id: ' => $request->input('data')['id']]);

        $fkStage = $this->pipeStage->getStageId($fkPipeline, $data['stage_id']);
        $contact = $this->syncPipeDriveContactToEvoTalks->syncPipeDriveContactToEvoTalks($data);
        $evoData = $this->pipeDriveToEvoTalksMapper->mapPipeDriveToEvoTalks($data, $dataV2, $fkPipeline, $fkStage, $contact);
        $response = $this->evoTalksController->createOpportunity($evoData);

        if ($response->status() === 200 && isset($response->getData()->id)) {

            $opportunityId = $response->getData()->id;
            $notes = $this->pipeDriveController->fetchNotes($data['id']);
            
            if ($notes) {
                $this->evoTalksController->addNotesToOpportunity(new Request([
                    'opportunity_id' => $opportunityId,
                    'notes' => $notes,
                ]));
                
                SlackNotifier::sendNotification('info', 'Adicionando notas à oportunidade no EvoTalks', ['Note' => $notes]);
            }

            $this->syncActivitiesFromPipeDrive(new Request(['deal_id' => $data['id']]), $opportunityId);
        } else {
            SlackNotifier::sendNotification('warning', 'Erro ao criar oportunidade no EvoTalks', ['response' => $response->json()]);
        }

        SlackNotifier::sendNotification('info', 'Oportunidade cadastrada no Evo');
        return $response;
    }

    public function syncActivitiesFromPipeDrive(Request $request, $opportunityId)
    {
        $dealId = $request->input('deal_id');

        if (!$dealId) {
            SlackNotifier::sendNotification('warning', 'ID do negócio (deal_id) não fornecido');
            return response()->json(['error' => 'deal_id is required'], 400);
        }

        $activities = $this->pipeDriveController->fetchDealFlowActivities($dealId);
        if (empty($activities)) {
            SlackNotifier::sendNotification('warning', 'Nenhuma atividade do tipo "activity" encontrada no Pipedrive', ['deal_id' => $dealId]);
            return response()->json(['message' => 'No activities found for the given deal'], 204);
        }

        foreach ($activities as $activity) {
            $this->syncPipeDriveActivitiesToEvoTalks->syncPipeDriveActivitiesToEvoTalks($activity, $opportunityId);
        }
        
        SlackNotifier::sendNotification('info', 'Tarefa criada com sucesso no EvoTalks', ['Total activities' => count($activities)]);

        return response()->json(['message' => 'Activities synced successfully'], 200);
    }

    private function isValidPayload($data)
    {
        return is_array($data) && isset($data['id'], $data['status'], $data['pipeline_id']);
    }  
}
